@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -283,15 +283,47 @@ function make_system_status($filter, $backurl) {
 
 	$table->setHeader($header);
 
-	// get host groups
-	$groups = API::HostGroup()->get([
-		'groupids' => $filter['groupids'],
-		'hostids' => isset($filter['hostids']) ? $filter['hostids'] : null,
-		'monitored_hosts' => true,
+	$options = [
 		'output' => ['groupid', 'name'],
+		'groupids' => $filter['groupids'],
+		'monitored_hosts' => true,
 		'preservekeys' => true
+	];
+
+	if (array_key_exists('hostids', $filter)) {
+		$options['hostids'] = $filter['hostids'];
+	}
+
+	$groups = API::HostGroup()->get($options);
+
+	$filter_groups = API::HostGroup()->get([
+		'output' => ['groupid', 'name'],
+		'groupids' => $filter['groupids']
 	]);
 
+	$filter_groups_names = [];
+
+	foreach ($filter_groups as $group) {
+		$filter_groups_names[] = $group['name'].'/';
+	}
+
+	$options = [
+		'output' => ['groupid', 'name'],
+		'monitored_hosts' => true,
+		'search' => ['name' => $filter_groups_names],
+		'startSearch' => true
+	];
+
+	if (array_key_exists('hostids', $filter)) {
+		$options['hostids'] = $filter['hostids'];
+	}
+
+	$child_groups = API::HostGroup()->get($options);
+
+	foreach ($child_groups as $child_group) {
+		$groups[$child_group['groupid']] = $child_group;
+	}
+
 	CArrayHelper::sort($groups, [
 		['field' => 'name', 'order' => ZBX_SORT_UP]
 	]);
@@ -316,7 +348,6 @@ function make_system_status($filter, $backurl) {
 		'output' => ['triggerid', 'priority', 'state', 'description', 'error', 'value', 'lastchange', 'expression'],
 		'selectGroups' => ['groupid'],
 		'selectHosts' => ['name'],
-		'selectLastEvent' => ['eventid', 'acknowledged', 'objectid'],
 		'withLastEventUnacknowledged' => ($filter['extAck'] == EXTACK_OPTION_UNACK) ? true : null,
 		'skipDependent' => true,
 		'groupids' => $groupIds,
@@ -333,34 +364,31 @@ function make_system_status($filter, $backurl) {
 		'preservekeys' => true
 	]);
 
-	$eventIds = [];
+	$problems = getTriggerLastProblems(array_keys($triggers), ['eventid', 'objectid', 'clock', 'acknowledged', 'ns']);
 
-	foreach ($triggers as $triggerId => $trigger) {
-		if ($trigger['lastEvent']) {
-			$eventIds[$trigger['lastEvent']['eventid']] = $trigger['lastEvent']['eventid'];
-		}
-
-		$triggers[$triggerId]['event'] = $trigger['lastEvent'];
-		unset($triggers[$triggerId]['lastEvent']);
+	$events = [];
+	foreach ($problems as $problem) {
+		$triggers[$problem['objectid']]['event'] = $problem;
+		$events[$problem['eventid']] = ['eventid' => $problem['eventid']];
 	}
 
 	// get acknowledges
-	if ($eventIds) {
+	if ($events) {
 		$eventAcknowledges = API::Event()->get([
 			'output' => ['eventid'],
-			'eventids' => $eventIds,
-			'select_acknowledges' => ['eventid', 'clock', 'message', 'alias', 'name', 'surname'],
+			'eventids' => array_keys($events),
+			'select_acknowledges' => ['eventid', 'clock', 'message', 'action', 'alias', 'name', 'surname'],
 			'preservekeys' => true
 		]);
 	}
 
 	// actions
-	$actions = makeEventsActions($eventIds);
+	$actions = makeEventsActions($events);
 
 	// triggers
 	foreach ($triggers as $trigger) {
 		// event
-		if ($trigger['event']) {
+		if (array_key_exists('event', $trigger)) {
 			$trigger['event']['acknowledges'] = isset($eventAcknowledges[$trigger['event']['eventid']])
 				? $eventAcknowledges[$trigger['event']['eventid']]['acknowledges']
 				: 0;
@@ -369,6 +397,7 @@ function make_system_status($filter, $backurl) {
 			$trigger['event'] = [
 				'acknowledged' => false,
 				'clock' => $trigger['lastchange'],
+				'ns' => '999999999',
 				'value' => $trigger['value']
 			];
 		}
@@ -508,10 +537,9 @@ function make_status_of_zbx() {
 	// check requirements
 	if (CWebUser::$data['type'] == USER_TYPE_SUPER_ADMIN) {
 		foreach ((new CFrontendSetup())->checkRequirements() as $req) {
-			if ($req['result'] != CFrontendSetup::CHECK_OK) {
-				$class = ($req['result'] == CFrontendSetup::CHECK_WARNING) ? ZBX_STYLE_ORANGE : ZBX_STYLE_RED;
+			if ($req['result'] == CFrontendSetup::CHECK_FATAL) {
 				$table->addRow(
-					(new CRow([$req['name'], $req['current'], $req['error']]))->addClass($class)
+					(new CRow([$req['name'], $req['current'], $req['error']]))->addClass(ZBX_STYLE_RED)
 				);
 			}
 		}
@@ -567,7 +595,6 @@ function make_latest_issues(array $filter = [], $backurl) {
 			'triggerid', 'expression', 'description', 'url', 'priority', 'lastchange', 'comments', 'error', 'state'
 		],
 		'selectHosts' => ['hostid'],
-		'selectLastEvent' => ['eventid', 'acknowledged', 'objectid', 'clock', 'ns'],
 		'withLastEventUnacknowledged' => (isset($filter['extAck']) && $filter['extAck'] == EXTACK_OPTION_UNACK)
 			? true
 			: null,
@@ -586,32 +613,35 @@ function make_latest_issues(array $filter = [], $backurl) {
 		'countOutput' => true
 	]));
 
+	$problems = getTriggerLastProblems(array_keys($triggers), ['eventid', 'objectid', 'clock', 'acknowledged', 'ns']);
+
+	$events = [];
+	foreach ($problems as $problem) {
+		$triggers[$problem['objectid']]['lastEvent'] = $problem;
+		$events[$problem['eventid']] = ['eventid' => $problem['eventid']];
+	}
+
 	// get acknowledges
 	$hostids = [];
-	$eventids = [];
 	foreach ($triggers as $trigger) {
 		foreach ($trigger['hosts'] as $host) {
 			$hostids[$host['hostid']] = true;
 		}
-
-		if ($trigger['lastEvent']) {
-			$eventids[] = $trigger['lastEvent']['eventid'];
-		}
 	}
 
 	$config = select_config();
 
-	if ($config['event_ack_enable'] && $eventids) {
+	if ($config['event_ack_enable'] && $events) {
 		$event_acknowledges = API::Event()->get([
 			'output' => ['eventid'],
-			'eventids' => $eventids,
-			'select_acknowledges' => API_OUTPUT_EXTEND,
+			'eventids' => array_keys($events),
+			'select_acknowledges' => ['clock', 'message', 'action', 'alias', 'name', 'surname'],
 			'preservekeys' => true
 		]);
 	}
 
 	// actions
-	$actions = makeEventsActions($eventids);
+	$actions = makeEventsActions($events);
 
 	// indicator of sort field
 	if ($show_sort_indicator) {
@@ -698,13 +728,13 @@ function make_latest_issues(array $filter = [], $backurl) {
 		array_pop($host_list);
 
 		// unknown triggers
-		$unknown = '';
+		$info_icons = [];
 		if ($trigger['state'] == TRIGGER_STATE_UNKNOWN) {
-			$unknown = makeUnknownIcon($trigger['error']);
+			$info_icons[] = makeUnknownIcon($trigger['error']);
 		}
 
 		// trigger has events
-		if ($trigger['lastEvent']) {
+		if (array_key_exists('lastEvent', $trigger)) {
 			// description
 			$description = CMacrosResolverHelper::resolveEventDescription(zbx_array_merge($trigger, [
 				'clock' => $trigger['lastEvent']['clock'],
@@ -721,7 +751,7 @@ function make_latest_issues(array $filter = [], $backurl) {
 		}
 
 		if ($config['event_ack_enable']) {
-			if ($trigger['lastEvent']) {
+			if (array_key_exists('lastEvent', $trigger)) {
 				$trigger['lastEvent']['acknowledges'] =
 					$event_acknowledges[$trigger['lastEvent']['eventid']]['acknowledges'];
 
@@ -735,7 +765,7 @@ function make_latest_issues(array $filter = [], $backurl) {
 		}
 
 		// description
-		if ($trigger['lastEvent'] || $trigger['comments'] !== '' || $trigger['url'] !== '') {
+		if (array_key_exists('lastEvent', $trigger) || $trigger['comments'] !== '' || $trigger['url'] !== '') {
 			$description = (new CSpan($description))
 				->setHint(make_popup_eventlist($trigger, $backurl), '', true, 'max-width: 500px')
 				->addClass(ZBX_STYLE_LINK_ACTION);
@@ -743,15 +773,15 @@ function make_latest_issues(array $filter = [], $backurl) {
 		$description = (new CCol($description))->addClass(getSeverityStyle($trigger['priority']));
 
 		// clock
-		$link_host = reset($trigger['hosts']);
 		$clock = new CLink(zbx_date2str(DATE_TIME_FORMAT_SECONDS, $trigger['lastchange']),
-			'events.php?filter_set=1&triggerid='.$trigger['triggerid'].'&source='.EVENT_SOURCE_TRIGGERS.
-				'&show_unknown=1&hostid='.$link_host['hostid'].'&stime='.date(TIMESTAMP_FORMAT, $trigger['lastchange']).
-				'&period='.ZBX_PERIOD_DEFAULT
+			(new CUrl('zabbix.php'))
+				->setArgument('action', 'problem.view')
+				->setArgument('filter_triggerids[]', $trigger['triggerid'])
+				->setArgument('filter_set', '1')
 		);
 
 		// actions
-		$action_hint = ($trigger['lastEvent'] && isset($actions[$trigger['lastEvent']['eventid']]))
+		$action_hint = (array_key_exists('lastEvent', $trigger) && isset($actions[$trigger['lastEvent']['eventid']]))
 			? $actions[$trigger['lastEvent']['eventid']]
 			: SPACE;
 
@@ -760,7 +790,7 @@ function make_latest_issues(array $filter = [], $backurl) {
 			$description,
 			$clock,
 			zbx_date2age($trigger['lastchange']),
-			$unknown,
+			makeInformationList($info_icons),
 			$ack,
 			(new CCol($action_hint))->addClass(ZBX_STYLE_NOWRAP)
 		]);
@@ -799,13 +829,16 @@ function makeTriggersPopup(array $triggers, $backurl, array $actions, array $con
 
 	CArrayHelper::sort($triggers, [['field' => 'lastchange', 'order' => ZBX_SORT_DOWN]]);
 
-	$triggers = CMacrosResolverHelper::resolveTriggerNames(zbx_toHash($triggers, 'triggerid'));
-
 	foreach ($triggers as $trigger) {
+		$description = CMacrosResolverHelper::resolveEventDescription(zbx_array_merge($trigger, [
+			'clock' => $trigger['event']['clock'],
+			'ns' => $trigger['event']['ns']
+		]));
+
 		// unknown triggers
-		$unknown = '';
+		$info_icons = [];
 		if ($trigger['state'] == TRIGGER_STATE_UNKNOWN) {
-			$unknown = makeUnknownIcon($trigger['error']);
+			$info_icons[] = makeUnknownIcon($trigger['error']);
 		}
 
 		// ack
@@ -825,9 +858,9 @@ function makeTriggersPopup(array $triggers, $backurl, array $actions, array $con
 
 		$popupTable->addRow([
 			$trigger['hosts'][0]['name'],
-			getSeverityCell($trigger['priority'], $config, $trigger['description']),
+			getSeverityCell($trigger['priority'], $config, $description),
 			zbx_date2age($trigger['lastchange']),
-			$unknown,
+			makeInformationList($info_icons),
 			$ack,
 			(new CCol($action))->addClass(ZBX_STYLE_NOWRAP)
 		]);
