@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #ifndef ZABBIX_COMMS_H
 #define ZABBIX_COMMS_H
 
-#if defined(_WINDOWS)
+#ifdef _WINDOWS
 #	if defined(__INT_MAX__) && __INT_MAX__ == 2147483647
 typedef int	ssize_t;
 #	else
@@ -28,36 +28,29 @@ typedef long	ssize_t;
 #	endif
 #endif
 
-#if defined(HAVE_POLARSSL)
-#	if defined(_WINDOWS) && defined(uint32_t)
-#		undef uint32_t
-#	endif
-#	include <polarssl/ssl.h>
-#elif defined(HAVE_GNUTLS)
-#	include <gnutls/gnutls.h>
-#elif defined(HAVE_OPENSSL)
-#	include <openssl/ssl.h>
-#endif
-
-#if defined(_WINDOWS)
+#ifdef _WINDOWS
 #	define ZBX_TCP_WRITE(s, b, bl)	((ssize_t)send((s), (b), (bl), 0))
 #	define ZBX_TCP_READ(s, b, bl)	((ssize_t)recv((s), (b), (bl), 0))
 #	define zbx_socket_close(s)	if (ZBX_SOCKET_ERROR != (s)) closesocket(s)
 #	define zbx_socket_last_error()	WSAGetLastError()
 
+#	define ZBX_PROTO_AGAIN		WSAEINTR
 #	define ZBX_PROTO_ERROR		SOCKET_ERROR
 #	define ZBX_SOCKET_ERROR		INVALID_SOCKET
+#	define ZBX_SOCKET_TO_INT(s)	((int)(s))
 #else
 #	define ZBX_TCP_WRITE(s, b, bl)	((ssize_t)write((s), (b), (bl)))
 #	define ZBX_TCP_READ(s, b, bl)	((ssize_t)read((s), (b), (bl)))
 #	define zbx_socket_close(s)	if (ZBX_SOCKET_ERROR != (s)) close(s)
 #	define zbx_socket_last_error()	errno
 
+#	define ZBX_PROTO_AGAIN		EINTR
 #	define ZBX_PROTO_ERROR		-1
 #	define ZBX_SOCKET_ERROR		-1
+#	define ZBX_SOCKET_TO_INT(s)	(s)
 #endif
 
-#if defined(SOCKET) || defined(_WINDOWS)
+#ifdef _WINDOWS
 typedef SOCKET	ZBX_SOCKET;
 #else
 typedef int	ZBX_SOCKET;
@@ -74,6 +67,10 @@ zbx_buf_type_t;
 #define ZBX_STAT_BUF_LEN	2048
 #define ZBX_SOCKET_PEER_BUF_LEN	129
 
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+typedef struct zbx_tls_context	zbx_tls_context_t;
+#endif
+
 typedef struct
 {
 	ZBX_SOCKET			socket;
@@ -81,14 +78,8 @@ typedef struct
 	size_t				read_bytes;
 	char				*buffer;
 	char				*next_line;
-#if defined(HAVE_POLARSSL)
-	ssl_context			*tls_ctx;
-#elif defined(HAVE_GNUTLS)
-	gnutls_session_t		tls_ctx;
-	gnutls_psk_client_credentials_t	tls_psk_client_creds;
-	gnutls_psk_server_credentials_t	tls_psk_server_creds;
-#elif defined(HAVE_OPENSSL)
-	SSL				*tls_ctx;
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_tls_context_t		*tls_ctx;
 #endif
 	unsigned int 			connection_type;	/* type of connection actually established: */
 								/* ZBX_TCP_SEC_UNENCRYPTED, ZBX_TCP_SEC_TLS_PSK or */
@@ -107,7 +98,7 @@ zbx_socket_t;
 
 const char	*zbx_socket_strerror(void);
 
-#if !defined(_WINDOWS)
+#ifndef _WINDOWS
 void	zbx_gethost_by_ip(const char *ip, char *host, size_t hostlen);
 #endif
 
@@ -123,6 +114,8 @@ int	zbx_tcp_connect(zbx_socket_t *s, const char *source_ip, const char *ip, unsi
 #define ZBX_TCP_SEC_TLS_PSK_TXT		"psk"
 #define ZBX_TCP_SEC_TLS_CERT_TXT	"cert"
 
+const char	*zbx_tcp_connection_type_name(unsigned int type);
+
 #define zbx_tcp_send(s, d)				zbx_tcp_send_ext((s), (d), strlen(d), ZBX_TCP_PROTOCOL, 0)
 #define zbx_tcp_send_to(s, d, timeout)			zbx_tcp_send_ext((s), (d), strlen(d), ZBX_TCP_PROTOCOL, timeout)
 #define zbx_tcp_send_bytes_to(s, d, len, timeout)	zbx_tcp_send_ext((s), (d), len, ZBX_TCP_PROTOCOL, timeout)
@@ -132,7 +125,7 @@ int	zbx_tcp_send_ext(zbx_socket_t *s, const char *data, size_t len, unsigned cha
 
 void	zbx_tcp_close(zbx_socket_t *s);
 
-#if defined(HAVE_IPV6)
+#ifdef HAVE_IPV6
 int	get_address_family(const char *addr, int *family, char *error, int max_error_len);
 #endif
 
@@ -185,10 +178,12 @@ int	zbx_send_response_ext(zbx_socket_t *sock, int result, const char *info, int
 
 int	zbx_recv_response(zbx_socket_t *sock, int timeout, char **error);
 
-#if defined(HAVE_IPV6)
-#define zbx_getnameinfo(sa, host, hostlen, serv, servlen, flags)						\
-	getnameinfo(sa, AF_INET == (sa)->sa_family ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6),	\
-		host, hostlen, serv, servlen, flags)
+#ifdef HAVE_IPV6
+#	define zbx_getnameinfo(sa, host, hostlen, serv, servlen, flags)		\
+			getnameinfo(sa, AF_INET == (sa)->sa_family ?		\
+					sizeof(struct sockaddr_in) :		\
+					sizeof(struct sockaddr_in6),		\
+					host, hostlen, serv, servlen, flags)
 #endif
 
 #endif
