@@ -24,11 +24,11 @@
 #define ZBX_STR(str)	ZBX_STR2(str)
 
 #define APPLICATION_NAME	"Zabbix Agent"
-#define ZABBIX_REVDATE		"4 May 2017"
+#define ZABBIX_REVDATE		"18 July 2017"
 #define ZABBIX_VERSION_MAJOR	3
 #define ZABBIX_VERSION_MINOR	2
-#define ZABBIX_VERSION_PATCH	6
-#define ZABBIX_VERSION_REVISION	67849
+#define ZABBIX_VERSION_PATCH	7
+#define ZABBIX_VERSION_REVISION	70298
 #define ZABBIX_VERSION_RC	""
 #define ZABBIX_VERSION		ZBX_STR(ZABBIX_VERSION_MAJOR) "." ZBX_STR(ZABBIX_VERSION_MINOR) "." \
 				ZBX_STR(ZABBIX_VERSION_PATCH) ZABBIX_VERSION_RC
