@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,6 +28,18 @@
 
 #define ZBX_DB_WAIT_DOWN	10
 
+typedef struct
+{
+	zbx_uint64_t	autoreg_hostid;
+	char		*host;
+	char		*ip;
+	char		*dns;
+	char		*host_metadata;
+	int		now;
+	unsigned short	port;
+}
+zbx_autoreg_host_t;
+
 #if HAVE_POSTGRESQL
 extern char	ZBX_PG_ESCAPE_BACKSLASH;
 #endif
@@ -223,67 +235,6 @@ void	DBstatement_prepare(const char *sql)
 		}
 	}
 }
-
-/******************************************************************************
- *                                                                            *
- * Function: DBbind_parameter                                                 *
- *                                                                            *
- * Purpose: creates an association between a program variable and             *
- *          a placeholder in a SQL statement                                  *
- *                                                                            *
- * Comments: retry until DB is up                                             *
- *                                                                            *
- ******************************************************************************/
-void	DBbind_parameter(int position, void *buffer, unsigned char type)
-{
-	int	rc;
-
-	rc = zbx_db_bind_parameter(position, buffer, type);
-
-	while (ZBX_DB_DOWN == rc)
-	{
-		DBclose();
-		DBconnect(ZBX_DB_CONNECT_NORMAL);
-
-		if (ZBX_DB_DOWN == (rc = zbx_db_bind_parameter(position, buffer, type)))
-		{
-			zabbix_log(LOG_LEVEL_ERR, "database is down: retrying in %d seconds", ZBX_DB_WAIT_DOWN);
-			connection_failure = 1;
-			sleep(ZBX_DB_WAIT_DOWN);
-		}
-	}
-}
-
-/******************************************************************************
- *                                                                            *
- * Function: DBstatement_execute                                              *
- *                                                                            *
- * Purpose: executes a SQL statement                                          *
- *                                                                            *
- * Comments: retry until DB is up                                             *
- *                                                                            *
- ******************************************************************************/
-int	DBstatement_execute()
-{
-	int	rc;
-
-	rc = zbx_db_statement_execute();
-
-	while (ZBX_DB_DOWN == rc)
-	{
-		DBclose();
-		DBconnect(ZBX_DB_CONNECT_NORMAL);
-
-		if (ZBX_DB_DOWN == (rc = zbx_db_statement_execute()))
-		{
-			zabbix_log(LOG_LEVEL_ERR, "database is down: retrying in %d seconds", ZBX_DB_WAIT_DOWN);
-			connection_failure = 1;
-			sleep(ZBX_DB_WAIT_DOWN);
-		}
-	}
-
-	return rc;
-}
 #endif
 
 /******************************************************************************
@@ -471,6 +422,54 @@ int	DBget_proxy_lastaccess(const char *hostname, int *lastaccess, char **error)
 	return ret;
 }
 
+#ifdef HAVE_MYSQL
+static size_t	get_string_field_size(unsigned char type)
+{
+	switch(type)
+	{
+		case ZBX_TYPE_LONGTEXT:
+			return ZBX_SIZE_T_MAX;
+		case ZBX_TYPE_CHAR:
+		case ZBX_TYPE_TEXT:
+		case ZBX_TYPE_SHORTTEXT:
+			return 65535u;
+		default:
+			THIS_SHOULD_NEVER_HAPPEN;
+			exit(EXIT_FAILURE);
+	}
+}
+#elif HAVE_ORACLE
+static size_t	get_string_field_size(unsigned char type)
+{
+	switch(type)
+	{
+		case ZBX_TYPE_LONGTEXT:
+		case ZBX_TYPE_TEXT:
+			return ZBX_SIZE_T_MAX;
+		case ZBX_TYPE_CHAR:
+		case ZBX_TYPE_SHORTTEXT:
+			return 4000u;
+		default:
+			THIS_SHOULD_NEVER_HAPPEN;
+			exit(EXIT_FAILURE);
+	}
+}
+#endif
+
+/******************************************************************************
+ *                                                                            *
+ * Function: DBdyn_escape_string_len                                          *
+ *                                                                            *
+ ******************************************************************************/
+char	*DBdyn_escape_string_len(const char *src, size_t length)
+{
+#if HAVE_IBM_DB2	/* IBM DB2 fields are limited by bytes rather than characters */
+	return zbx_db_dyn_escape_string(src, length, ZBX_SIZE_T_MAX, ESCAPE_SEQUENCE_ON);
+#else
+	return zbx_db_dyn_escape_string(src, ZBX_SIZE_T_MAX, length, ESCAPE_SEQUENCE_ON);
+#endif
+}
+
 /******************************************************************************
  *                                                                            *
  * Function: DBdyn_escape_string                                              *
@@ -478,17 +477,49 @@ int	DBget_proxy_lastaccess(const char *hostname, int *lastaccess, char **error)
  ******************************************************************************/
 char	*DBdyn_escape_string(const char *src)
 {
-	return zbx_db_dyn_escape_string(src);
+	return zbx_db_dyn_escape_string(src, ZBX_SIZE_T_MAX, ZBX_SIZE_T_MAX, ESCAPE_SEQUENCE_ON);
 }
 
 /******************************************************************************
  *                                                                            *
- * Function: DBdyn_escape_string_len                                          *
+ * Function: DBdyn_escape_field_len                                           *
  *                                                                            *
  ******************************************************************************/
-char	*DBdyn_escape_string_len(const char *src, size_t max_src_len)
+static char	*DBdyn_escape_field_len(const ZBX_FIELD *field, const char *src, zbx_escape_sequence_t flag)
 {
-	return zbx_db_dyn_escape_string_len(src, max_src_len);
+	size_t	length;
+
+	if (ZBX_TYPE_LONGTEXT == field->type && 0 == field->length)
+		length = ZBX_SIZE_T_MAX;
+	else
+		length = field->length;
+
+#if defined(HAVE_MYSQL) || defined(HAVE_ORACLE)
+	return zbx_db_dyn_escape_string(src, get_string_field_size(field->type), length, flag);
+#elif HAVE_IBM_DB2	/* IBM DB2 fields are limited by bytes rather than characters */
+	return zbx_db_dyn_escape_string(src, length, ZBX_SIZE_T_MAX, flag);
+#else
+	return zbx_db_dyn_escape_string(src, ZBX_SIZE_T_MAX, length, flag);
+#endif
+}
+
+/******************************************************************************
+ *                                                                            *
+ * Function: DBdyn_escape_field                                               *
+ *                                                                            *
+ ******************************************************************************/
+char	*DBdyn_escape_field(const char *table_name, const char *field_name, const char *src)
+{
+	const ZBX_TABLE	*table;
+	const ZBX_FIELD	*field;
+
+	if (NULL == (table = DBget_table(table_name)) || NULL == (field = DBget_field(table, field_name)))
+	{
+		zabbix_log(LOG_LEVEL_CRIT, "invalid table: \"%s\" field: \"%s\"", table_name, field_name);
+		exit(EXIT_FAILURE);
+	}
+
+	return DBdyn_escape_field_len(field, src, ESCAPE_SEQUENCE_ON);
 }
 
 /******************************************************************************
@@ -853,6 +884,9 @@ void	DBadd_condition_alloc(char **sql, size_t *sql_alloc, size_t *sql_offset, co
  *             values     - [IN] array of string values                       *
  *             num        - [IN] number of elements in 'values' array         *
  *                                                                            *
+ * Comments: To support Oracle empty values are checked separately (is null   *
+ *           for Oracle and ='' for the other databases).                     *
+ *                                                                            *
  ******************************************************************************/
 void	DBadd_str_condition_alloc(char **sql, size_t *sql_alloc, size_t *sql_offset, const char *fieldname,
 		const char **values, const int num)
@@ -861,29 +895,59 @@ void	DBadd_str_condition_alloc(char **sql, size_t *sql_alloc, size_t *sql_offset
 
 	int	i, cnt = 0;
 	char	*value_esc;
+	int	values_num = 0, empty_num = 0;
 
 	if (0 == num)
 		return;
 
 	zbx_chrcpy_alloc(sql, sql_alloc, sql_offset, ' ');
 
-	if (1 == num)
+	for (i = 0; i < num; i++)
 	{
-		value_esc = DBdyn_escape_string(values[0]);
-		zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s='%s'", fieldname, value_esc);
-		zbx_free(value_esc);
-
-		return;
+		if ('\0' == *values[i])
+			empty_num++;
+		else
+			values_num++;
 	}
 
-	if (MAX_EXPRESSIONS < num)
+	if (MAX_EXPRESSIONS < values_num || (0 != values_num && 0 != empty_num))
 		zbx_chrcpy_alloc(sql, sql_alloc, sql_offset, '(');
 
+	if (0 != empty_num)
+	{
+		zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s" ZBX_SQL_STRCMP, fieldname, ZBX_SQL_STRVAL_EQ(""));
+
+		if (0 == values_num)
+			return;
+
+		zbx_strcpy_alloc(sql, sql_alloc, sql_offset, " or ");
+	}
+
+	if (1 == values_num)
+	{
+		for (i = 0; i < num; i++)
+		{
+			if ('\0' == *values[i])
+				continue;
+
+			value_esc = DBdyn_escape_string(values[i]);
+			zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%s='%s'", fieldname, value_esc);
+			zbx_free(value_esc);
+		}
+
+		if (0 != empty_num)
+			zbx_chrcpy_alloc(sql, sql_alloc, sql_offset, ')');
+		return;
+	}
+
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, fieldname);
 	zbx_strcpy_alloc(sql, sql_alloc, sql_offset, " in (");
 
 	for (i = 0; i < num; i++)
 	{
+		if ('\0' == *values[i])
+			continue;
+
 		if (MAX_EXPRESSIONS == cnt)
 		{
 			cnt = 0;
@@ -905,7 +969,7 @@ void	DBadd_str_condition_alloc(char **sql, size_t *sql_alloc, size_t *sql_offset
 	(*sql_offset)--;
 	zbx_chrcpy_alloc(sql, sql_alloc, sql_offset, ')');
 
-	if (MAX_EXPRESSIONS < num)
+	if (MAX_EXPRESSIONS < values_num || 0 != empty_num)
 		zbx_chrcpy_alloc(sql, sql_alloc, sql_offset, ')');
 
 #undef MAX_EXPRESSIONS
@@ -1042,77 +1106,286 @@ const char	*DBsql_id_cmp(zbx_uint64_t id)
 void	DBregister_host(zbx_uint64_t proxy_hostid, const char *host, const char *ip, const char *dns,
 		unsigned short port, const char *host_metadata, int now)
 {
-	char		*host_esc, *ip_esc, *dns_esc, *host_metadata_esc;
+	zbx_vector_ptr_t	autoreg_hosts;
+
+	zbx_vector_ptr_create(&autoreg_hosts);
+
+	DBregister_host_prepare(&autoreg_hosts, host, ip, dns, port, host_metadata, now);
+	DBregister_host_flush(&autoreg_hosts, proxy_hostid);
+
+	DBregister_host_clean(&autoreg_hosts);
+	zbx_vector_ptr_destroy(&autoreg_hosts);
+}
+
+static int	DBregister_host_active(void)
+{
+	const char	*__function_name = "DBregister_host_active";
+
 	DB_RESULT	result;
-	DB_ROW		row;
-	zbx_uint64_t	autoreg_hostid;
-	zbx_timespec_t	ts;
-	int		res = SUCCEED;
+	int		ret = SUCCEED;
 
-	host_esc = DBdyn_escape_string_len(host, HOST_HOST_LEN);
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
-	ts.sec = now;
-	ts.ns = 0;
+	result = DBselect(
+			"select null"
+			" from actions"
+			" where eventsource=%d"
+				" and status=%d",
+			EVENT_SOURCE_AUTO_REGISTRATION,
+			ACTION_STATUS_ACTIVE);
+
+	if (NULL == DBfetch(result))
+		ret = FAIL;
+
+	DBfree_result(result);
+
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s():%s", __function_name, zbx_result_string(ret));
+
+	return ret;
+}
+
+static void	autoreg_host_free(zbx_autoreg_host_t *autoreg_host)
+{
+	zbx_free(autoreg_host->host);
+	zbx_free(autoreg_host->ip);
+	zbx_free(autoreg_host->dns);
+	zbx_free(autoreg_host->host_metadata);
+	zbx_free(autoreg_host);
+}
+
+void	DBregister_host_prepare(zbx_vector_ptr_t *autoreg_hosts, const char *host, const char *ip, const char *dns,
+		unsigned short port, const char *host_metadata, int now)
+{
+	zbx_autoreg_host_t	*autoreg_host;
+	int 			i;
+
+	for (i = 0; i < autoreg_hosts->values_num; i++)	/* duplicate check */
+	{
+		autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
+
+		if (0 == strcmp(host, autoreg_host->host))
+		{
+			zbx_vector_ptr_remove(autoreg_hosts, i);
+			autoreg_host_free(autoreg_host);
+			break;
+		}
+	}
+
+	autoreg_host = (zbx_autoreg_host_t *)zbx_malloc(NULL, sizeof(zbx_autoreg_host_t));
+	autoreg_host->autoreg_hostid = 0;
+	autoreg_host->host = zbx_strdup(NULL, host);
+	autoreg_host->ip = zbx_strdup(NULL, ip);
+	autoreg_host->dns = zbx_strdup(NULL, dns);
+	autoreg_host->port = port;
+	autoreg_host->host_metadata = zbx_strdup(NULL, host_metadata);
+	autoreg_host->now = now;
+
+	zbx_vector_ptr_append(autoreg_hosts, autoreg_host);
+}
+
+static void	autoreg_get_hosts(zbx_vector_ptr_t *autoreg_hosts, zbx_vector_str_t *hosts)
+{
+	int	i;
+
+	for (i = 0; i < autoreg_hosts->values_num; i++)
+	{
+		zbx_autoreg_host_t	*autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
+
+		zbx_vector_str_append(hosts, autoreg_host->host);
+	}
+}
+
+static void	process_autoreg_hosts(zbx_vector_ptr_t *autoreg_hosts, zbx_uint64_t proxy_hostid)
+{
+	DB_RESULT		result;
+	DB_ROW			row;
+	zbx_vector_str_t	hosts;
+	char			*sql = NULL;
+	size_t			sql_alloc = 256, sql_offset;
+	zbx_autoreg_host_t	*autoreg_host;
+	int			i;
+
+	sql = zbx_malloc(sql, sql_alloc);
+	zbx_vector_str_create(&hosts);
 
 	if (0 != proxy_hostid)
 	{
-		result = DBselect(
-				"select hostid"
+		autoreg_get_hosts(autoreg_hosts, &hosts);
+
+		/* delete from vector if already exist in hosts table */
+		sql_offset = 0;
+		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+				"select host"
 				" from hosts"
 				" where proxy_hostid=" ZBX_FS_UI64
-					" and host='%s'",
-				proxy_hostid, host_esc);
+					" and",
+				proxy_hostid);
+		DBadd_str_condition_alloc(&sql, &sql_alloc, &sql_offset, "host",
+				(const char **)hosts.values, hosts.values_num);
 
-		if (NULL != DBfetch(result))
-			res = FAIL;
+		result = DBselect("%s", sql);
+
+		while (NULL != (row = DBfetch(result)))
+		{
+			for (i = 0; i < autoreg_hosts->values_num; i++)
+			{
+				autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
+
+				if (0 == strcmp(autoreg_host->host, row[0]))
+				{
+					zbx_vector_ptr_remove(autoreg_hosts, i);
+					autoreg_host_free(autoreg_host);
+					break;
+				}
+			}
+
+		}
 		DBfree_result(result);
+
+		hosts.values_num = 0;
 	}
 
-	if (SUCCEED == res)
+	if (0 != autoreg_hosts->values_num)
 	{
-		ip_esc = DBdyn_escape_string_len(ip, INTERFACE_IP_LEN);
-		dns_esc = DBdyn_escape_string_len(dns, INTERFACE_DNS_LEN);
-		host_metadata_esc = DBdyn_escape_string(host_metadata);
+		autoreg_get_hosts(autoreg_hosts, &hosts);
 
-		result = DBselect(
-				"select autoreg_hostid"
+		/* update autoreg_id in vector if already exists in autoreg_host table */
+		sql_offset = 0;
+		zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+				"select autoreg_hostid,host"
 				" from autoreg_host"
 				" where proxy_hostid%s"
-					" and host='%s'",
-				DBsql_id_cmp(proxy_hostid), host_esc);
+					" and",
+				DBsql_id_cmp(proxy_hostid));
+		DBadd_str_condition_alloc(&sql, &sql_alloc, &sql_offset, "host",
+				(const char **)hosts.values, hosts.values_num);
 
-		if (NULL != (row = DBfetch(result)))
+		result = DBselect("%s", sql);
+
+		while (NULL != (row = DBfetch(result)))
 		{
-			ZBX_STR2UINT64(autoreg_hostid, row[0]);
+			for (i = 0; i < autoreg_hosts->values_num; i++)
+			{
+				autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
 
-			DBexecute("update autoreg_host"
-					" set listen_ip='%s',listen_dns='%s',listen_port=%d,host_metadata='%s'"
-					" where autoreg_hostid=" ZBX_FS_UI64,
-					ip_esc, dns_esc, (int)port, host_metadata_esc, autoreg_hostid);
+				if (0 == autoreg_host->autoreg_hostid && 0 == strcmp(autoreg_host->host, row[1]))
+				{
+					ZBX_STR2UINT64(autoreg_host->autoreg_hostid, row[0]);
+					break;
+				}
+			}
+		}
+		DBfree_result(result);
+
+		hosts.values_num = 0;
+	}
+
+	zbx_vector_str_destroy(&hosts);
+	zbx_free(sql);
+}
+
+void	DBregister_host_flush(zbx_vector_ptr_t *autoreg_hosts, zbx_uint64_t proxy_hostid)
+{
+	const char		*__function_name = "DBregister_host_flush";
+
+	zbx_autoreg_host_t	*autoreg_host;
+	zbx_uint64_t		autoreg_hostid;
+	zbx_db_insert_t		db_insert;
+	int			i, new = 0, upd = 0;
+	char			*sql = NULL, *ip_esc, *dns_esc, *host_metadata_esc;
+	size_t			sql_alloc = 256, sql_offset = 0;
+	zbx_timespec_t		ts = {0, 0};
+
+	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
+
+	if (SUCCEED != DBregister_host_active())
+		goto exit;
+
+	process_autoreg_hosts(autoreg_hosts, proxy_hostid);
+
+	for (i = 0; i < autoreg_hosts->values_num; i++)
+	{
+		autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
+
+		if (0 == autoreg_host->autoreg_hostid)
+			new++;
+	}
+
+	if (0 != new)
+	{
+		autoreg_hostid = DBget_maxid_num("autoreg_host", new);
+
+		zbx_db_insert_prepare(&db_insert, "autoreg_host", "autoreg_hostid", "proxy_hostid", "host", "listen_ip",
+				"listen_dns", "listen_port", "host_metadata", NULL);
+	}
+
+	if (0 != (upd = autoreg_hosts->values_num - new))
+	{
+		sql = zbx_malloc(sql, sql_alloc);
+		DBbegin_multiple_update(&sql, &sql_alloc, &sql_offset);
+	}
+
+	zbx_vector_ptr_sort(autoreg_hosts, ZBX_DEFAULT_UINT64_PTR_COMPARE_FUNC);
+
+	for (i = 0; i < autoreg_hosts->values_num; i++)
+	{
+		autoreg_host = (zbx_autoreg_host_t *)autoreg_hosts->values[i];
+
+		if (0 == autoreg_host->autoreg_hostid)
+		{
+			autoreg_host->autoreg_hostid = autoreg_hostid++;
+
+			zbx_db_insert_add_values(&db_insert, autoreg_host->autoreg_hostid, proxy_hostid,
+					autoreg_host->host, autoreg_host->ip, autoreg_host->dns,
+					(int)autoreg_host->port, autoreg_host->host_metadata);
 		}
 		else
 		{
-			autoreg_hostid = DBget_maxid("autoreg_host");
-			DBexecute("insert into autoreg_host"
-					" (autoreg_hostid,proxy_hostid,host,listen_ip,listen_dns,listen_port,"
-						"host_metadata)"
-					" values"
-					" (" ZBX_FS_UI64 ",%s,'%s','%s','%s',%d,'%s')",
-					autoreg_hostid, DBsql_id_ins(proxy_hostid),
-					host_esc, ip_esc, dns_esc, (int)port, host_metadata_esc);
+			ip_esc = DBdyn_escape_string(autoreg_host->ip);
+			dns_esc = DBdyn_escape_string(autoreg_host->dns);
+			host_metadata_esc = DBdyn_escape_string(autoreg_host->host_metadata);
+
+			zbx_snprintf_alloc(&sql, &sql_alloc, &sql_offset,
+					"update autoreg_host"
+					" set listen_ip='%s',"
+						"listen_dns='%s',"
+						"listen_port=%hu,"
+						"host_metadata='%s'"
+					" where autoreg_hostid=" ZBX_FS_UI64 ";\n",
+				ip_esc, dns_esc, autoreg_host->port, host_metadata_esc, autoreg_host->autoreg_hostid);
+
+			zbx_free(host_metadata_esc);
+			zbx_free(dns_esc);
+			zbx_free(ip_esc);
 		}
-		DBfree_result(result);
 
-		zbx_free(host_metadata_esc);
-		zbx_free(dns_esc);
-		zbx_free(ip_esc);
+		ts.sec = autoreg_host->now;
 
-		add_event(EVENT_SOURCE_AUTO_REGISTRATION, EVENT_OBJECT_ZABBIX_ACTIVE, autoreg_hostid, &ts,
-				TRIGGER_VALUE_PROBLEM, NULL, NULL, NULL, 0, 0, NULL, 0, NULL);
-		process_events();
+		add_event(EVENT_SOURCE_AUTO_REGISTRATION, EVENT_OBJECT_ZABBIX_ACTIVE, autoreg_host->autoreg_hostid,
+				&ts, TRIGGER_VALUE_PROBLEM, NULL, NULL, NULL, 0, 0, NULL, 0, NULL);
 	}
 
-	zbx_free(host_esc);
+	if (0 != new)
+	{
+		zbx_db_insert_execute(&db_insert);
+		zbx_db_insert_clean(&db_insert);
+	}
+
+	if (0 != upd)
+	{
+		DBend_multiple_update(&sql, &sql_alloc, &sql_offset);
+		DBexecute("%s", sql);
+		zbx_free(sql);
+	}
+
+	process_events();
+exit:
+	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+}
+
+void	DBregister_host_clean(zbx_vector_ptr_t *autoreg_hosts)
+{
+	zbx_vector_ptr_clear_ext(autoreg_hosts, (zbx_mem_free_func_t)autoreg_host_free);
 }
 
 /******************************************************************************
@@ -1131,10 +1404,10 @@ void	DBproxy_register_host(const char *host, const char *ip, const char *dns, un
 {
 	char	*host_esc, *ip_esc, *dns_esc, *host_metadata_esc;
 
-	host_esc = DBdyn_escape_string_len(host, HOST_HOST_LEN);
-	ip_esc = DBdyn_escape_string_len(ip, INTERFACE_IP_LEN);
-	dns_esc = DBdyn_escape_string_len(dns, INTERFACE_DNS_LEN);
-	host_metadata_esc = DBdyn_escape_string(host_metadata);
+	host_esc = DBdyn_escape_field("proxy_autoreg_host", "host", host);
+	ip_esc = DBdyn_escape_field("proxy_autoreg_host", "listen_ip", ip);
+	dns_esc = DBdyn_escape_field("proxy_autoreg_host", "listen_dns", dns);
+	host_metadata_esc = DBdyn_escape_field("proxy_autoreg_host", "host_metadata", host_metadata);
 
 	DBexecute("insert into proxy_autoreg_host"
 			" (clock,host,listen_ip,listen_dns,listen_port,host_metadata)"
@@ -1351,51 +1624,6 @@ const char	*DBget_inventory_field(unsigned char inventory_link)
 	return inventory_fields[inventory_link - 1];
 }
 
-/******************************************************************************
- *                                                                            *
- * Function: DBget_inventory_field_len                                        *
- *                                                                            *
- * Purpose: get host_inventory field length by inventory_link                 *
- *                                                                            *
- * Parameters: inventory_link - [IN] field number; 1..ZBX_MAX_INVENTORY_FIELDS*
- *                                                                            *
- * Return value: field length                                                 *
- *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
- ******************************************************************************/
-unsigned short	DBget_inventory_field_len(unsigned char inventory_link)
-{
-	static unsigned short	*inventory_field_len = NULL;
-	const char		*inventory_field;
-	const ZBX_TABLE		*table;
-	const ZBX_FIELD		*field;
-
-	if (1 > inventory_link || inventory_link > ZBX_MAX_INVENTORY_FIELDS)
-		assert(0);
-
-	inventory_link--;
-
-	if (NULL == inventory_field_len)
-	{
-		inventory_field_len = zbx_malloc(inventory_field_len, ZBX_MAX_INVENTORY_FIELDS * sizeof(unsigned short));
-		memset(inventory_field_len, 0, ZBX_MAX_INVENTORY_FIELDS * sizeof(unsigned short));
-	}
-
-	if (0 != inventory_field_len[inventory_link])
-		return inventory_field_len[inventory_link];
-
-	inventory_field = DBget_inventory_field(inventory_link + 1);
-	table = DBget_table("host_inventory");
-	assert(NULL != table);
-	field = DBget_field(table, inventory_field);
-	assert(NULL != field);
-
-	inventory_field_len[inventory_link] = field->length;
-
-	return inventory_field_len[inventory_link];
-}
-
 #undef ZBX_MAX_INVENTORY_FIELDS
 
 int	DBtxn_status(void)
@@ -1743,9 +1971,6 @@ void	zbx_db_insert_clean(zbx_db_insert_t *self)
  *             fields      - [IN] names of the fields to insert               *
  *             fields_num  - [IN] the number of items in fields array         *
  *                                                                            *
- * Return value: Returns SUCCEED if the operation completed successfully or   *
- *               FAIL otherwise.                                              *
- *                                                                            *
  * Comments: The operation fails if the target table does not have the        *
  *           specified fields defined in its schema.                          *
  *                                                                            *
@@ -1863,31 +2088,17 @@ void	zbx_db_insert_add_values_dyn(zbx_db_insert_t *self, const zbx_db_value_t **
 	{
 		ZBX_FIELD		*field = self->fields.values[i];
 		const zbx_db_value_t	*value = values[i];
-#ifdef HAVE_ORACLE
-		size_t			str_alloc = 0, str_offset = 0;
-#endif
+
 		switch (field->type)
 		{
 			case ZBX_TYPE_LONGTEXT:
-				if (0 == field->length)
-				{
-#ifdef HAVE_ORACLE
-					row[i].str = zbx_strdup(NULL, value->str);
-#else
-					row[i].str = DBdyn_escape_string(value->str);
-#endif
-					break;
-				}
-				/* break; is not missing here */
 			case ZBX_TYPE_CHAR:
 			case ZBX_TYPE_TEXT:
 			case ZBX_TYPE_SHORTTEXT:
 #ifdef HAVE_ORACLE
-				row[i].str = NULL;
-				zbx_strncpy_alloc(&row[i].str, &str_alloc, &str_offset, value->str,
-						zbx_strlen_utf8_nchars(value->str, field->length));
+				row[i].str = DBdyn_escape_field_len(field, value->str, ESCAPE_SEQUENCE_OFF);
 #else
-				row[i].str = DBdyn_escape_string_len(value->str, field->length);
+				row[i].str = DBdyn_escape_field_len(field, value->str, ESCAPE_SEQUENCE_ON);
 #endif
 				break;
 			default:
@@ -1908,9 +2119,6 @@ void	zbx_db_insert_add_values_dyn(zbx_db_insert_t *self, const zbx_db_value_t **
  * Parameters: self - [IN] the bulk insert data                               *
  *             ...  - [IN] the values to insert                               *
  *                                                                            *
- * Return value: Returns SUCCEED if the operation completed successfully or   *
- *               FAIL otherwise.                                              *
- *                                                                            *
  * Comments: This is a convenience wrapper for zbx_db_insert_add_values_dyn() *
  *           function.                                                        *
  *           Note that the types of the passed values must conform to the     *
@@ -1996,6 +2204,9 @@ int	zbx_db_insert_execute(zbx_db_insert_t *self)
 	char		*sql_values = NULL;
 	size_t		sql_values_alloc = 0, sql_values_offset = 0;
 #	endif
+#else
+	zbx_db_bind_context_t	*contexts;
+	int			rc, tries = 0;
 #endif
 
 	if (0 == self->rows.values_num)
@@ -2069,51 +2280,59 @@ int	zbx_db_insert_execute(zbx_db_insert_t *self)
 	}
 	zbx_chrcpy_alloc(&sql_command, &sql_command_alloc, &sql_command_offset, ')');
 
+	contexts = (zbx_db_bind_context_t *)zbx_malloc(NULL, sizeof(zbx_db_bind_context_t) * self->fields.values_num);
+
+retry_oracle:
 	DBstatement_prepare(sql_command);
 
-	for (i = 0; i < self->rows.values_num; i++)
+	for (j = 0; j < self->fields.values_num; j++)
 	{
-		zbx_db_value_t	*values = (zbx_db_value_t *)self->rows.values[i];
+		field = (ZBX_FIELD *)self->fields.values[j];
 
-		if (SUCCEED == zabbix_check_log_level(LOG_LEVEL_DEBUG))
+		if (ZBX_DB_OK > zbx_db_bind_parameter_dyn(&contexts[j], j, field->type,
+				(zbx_db_value_t **)self->rows.values, self->rows.values_num))
 		{
+			for (i = 0; i < j; i++)
+				zbx_db_clean_bind_context(&contexts[i]);
+
+			goto out;
+		}
+	}
+
+	if (SUCCEED == zabbix_check_log_level(LOG_LEVEL_DEBUG))
+	{
+		for (i = 0; i < self->rows.values_num; i++)
+		{
+			zbx_db_value_t	*values = (zbx_db_value_t *)self->rows.values[i];
 			char	*str;
 
 			str = zbx_db_format_values((ZBX_FIELD **)self->fields.values, values, self->fields.values_num);
 			zabbix_log(LOG_LEVEL_DEBUG, "insert [txnlev:%d] [%s]", zbx_db_txn_level(), str);
 			zbx_free(str);
 		}
-
-		for (j = 0; j < self->fields.values_num; j++)
-		{
-			const zbx_db_value_t	*value = &values[j];
-
-			field = self->fields.values[j];
-
-			switch (field->type)
-			{
-				case ZBX_TYPE_CHAR:
-				case ZBX_TYPE_TEXT:
-				case ZBX_TYPE_SHORTTEXT:
-				case ZBX_TYPE_LONGTEXT:
-					DBbind_parameter(j + 1, (void *)value->str, field->type);
-					break;
-				default:
-					DBbind_parameter(j + 1, (void *)value, field->type);
-					break;
-			}
-
-			if (0 != zbx_db_txn_error())
-			{
-				zabbix_log(LOG_LEVEL_ERR, "failed to bind field: %s", field->name);
-				goto out;
-			}
-		}
-		if (ZBX_DB_OK > DBstatement_execute())
-			goto out;
-
 	}
-	ret = SUCCEED;
+
+	rc = zbx_db_statement_execute(self->rows.values_num);
+
+	for (j = 0; j < self->fields.values_num; j++)
+		zbx_db_clean_bind_context(&contexts[j]);
+
+	if (ZBX_DB_DOWN == rc)
+	{
+		if (0 < tries++)
+		{
+			zabbix_log(LOG_LEVEL_ERR, "database is down: retrying in %d seconds", ZBX_DB_WAIT_DOWN);
+			connection_failure = 1;
+			sleep(ZBX_DB_WAIT_DOWN);
+		}
+
+		DBclose();
+		DBconnect(ZBX_DB_CONNECT_NORMAL);
+
+		goto retry_oracle;
+	}
+
+	ret = (ZBX_DB_OK <= rc ? SUCCEED : FAIL);
 
 #else
 	DBbegin_multiple_update(&sql, &sql_alloc, &sql_offset);
@@ -2203,6 +2422,8 @@ out:
 #	ifdef HAVE_MYSQL
 	zbx_free(sql_values);
 #	endif
+#else
+	zbx_free(contexts);
 #endif
 	return ret;
 }
@@ -2215,9 +2436,6 @@ out:
  *                                                                            *
  * Parameters: self - [IN] the bulk insert data                               *
  *                                                                            *
- * Return value: Returns SUCCEED if the operation completed successfully or   *
- *               FAIL otherwise.                                              *
- *                                                                            *
  ******************************************************************************/
 void	zbx_db_insert_autoincrement(zbx_db_insert_t *self, const char *field_name)
 {
@@ -2444,7 +2662,7 @@ int	zbx_sql_add_host_availability(char **sql, size_t *sql_alloc, size_t *sql_off
 		{
 			char	*error_esc;
 
-			error_esc = DBdyn_escape_string_len(ha->agents[i].error, HOST_ERROR_LEN);
+			error_esc = DBdyn_escape_field("hosts", "error", ha->agents[i].error);
 			zbx_snprintf_alloc(sql, sql_alloc, sql_offset, "%c%serror='%s'", delim, field_prefix[i],
 					error_esc);
 			zbx_free(error_esc);
@@ -2470,4 +2688,3 @@ int	zbx_sql_add_host_availability(char **sql, size_t *sql_alloc, size_t *sql_off
 
 	return SUCCEED;
 }
-
