@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -345,14 +345,14 @@ static void	lld_gitems_get(zbx_uint64_t parent_graphid, zbx_vector_ptr_t *gitems
  *             items             - [OUT] sorted list of items                 *
  *                                                                            *
  ******************************************************************************/
-static void	lld_items_get(zbx_vector_ptr_t *gitems_proto, zbx_uint64_t ymin_itemid_proto,
+static void	lld_items_get(const zbx_vector_ptr_t *gitems_proto, zbx_uint64_t ymin_itemid_proto,
 		zbx_uint64_t ymax_itemid_proto, zbx_vector_ptr_t *items)
 {
 	const char		*__function_name = "lld_items_get";
 
 	DB_RESULT		result;
 	DB_ROW			row;
-	zbx_lld_gitem_t		*gitem;
+	const zbx_lld_gitem_t	*gitem;
 	zbx_lld_item_t		*item;
 	zbx_vector_uint64_t	itemids;
 	int			i;
@@ -456,14 +456,14 @@ static zbx_lld_graph_t	*lld_graph_by_item(zbx_vector_ptr_t *graphs, zbx_uint64_t
  * Return value: upon successful completion return pointer to the graph       *
  *                                                                            *
  ******************************************************************************/
-static zbx_lld_graph_t	*lld_graph_get(zbx_vector_ptr_t *graphs, zbx_vector_ptr_t *item_links)
+static zbx_lld_graph_t	*lld_graph_get(zbx_vector_ptr_t *graphs, const zbx_vector_ptr_t *item_links)
 {
 	int		i;
 	zbx_lld_graph_t	*graph;
 
 	for (i = 0; i < item_links->values_num; i++)
 	{
-		zbx_lld_item_link_t	*item_link = (zbx_lld_item_link_t *)item_links->values[i];
+		const zbx_lld_item_link_t	*item_link = (zbx_lld_item_link_t *)item_links->values[i];
 
 		if (NULL != (graph = lld_graph_by_item(graphs, item_link->itemid)))
 			return graph;
@@ -482,8 +482,8 @@ static zbx_lld_graph_t	*lld_graph_get(zbx_vector_ptr_t *graphs, zbx_vector_ptr_t
  * Return value: SUCCEED if item successfully processed, FAIL - otherwise     *
  *                                                                            *
  ******************************************************************************/
-static int	lld_item_get(zbx_uint64_t itemid_proto, zbx_vector_ptr_t *items, zbx_vector_ptr_t *item_links,
-		zbx_uint64_t *itemid)
+static int	lld_item_get(zbx_uint64_t itemid_proto, const zbx_vector_ptr_t *items,
+		const zbx_vector_ptr_t *item_links, zbx_uint64_t *itemid)
 {
 	int			index;
 	zbx_lld_item_t		*item_proto;
@@ -511,13 +511,14 @@ static int	lld_item_get(zbx_uint64_t itemid_proto, zbx_vector_ptr_t *items, zbx_
 	return SUCCEED;
 }
 
-static int	lld_gitems_make(zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *gitems, zbx_vector_ptr_t *items,
-		zbx_vector_ptr_t *item_links)
+static int	lld_gitems_make(const zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *gitems,
+		const zbx_vector_ptr_t *items, const zbx_vector_ptr_t *item_links)
 {
 	const char		*__function_name = "lld_gitems_make";
 
 	int			i, ret = FAIL;
-	zbx_lld_gitem_t		*gitem_proto, *gitem;
+	const zbx_lld_gitem_t	*gitem_proto;
+	zbx_lld_gitem_t		*gitem;
 	zbx_uint64_t		itemid;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
@@ -617,16 +618,16 @@ out:
  * Purpose: create a graph based on lld rule and add it to the list           *
  *                                                                            *
  ******************************************************************************/
-static void 	lld_graph_make(zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *graphs, zbx_vector_ptr_t *items,
+static void 	lld_graph_make(const zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *graphs, zbx_vector_ptr_t *items,
 		const char *name_proto, zbx_uint64_t ymin_itemid_proto, zbx_uint64_t ymax_itemid_proto,
-		zbx_lld_row_t *lld_row)
+		const zbx_lld_row_t *lld_row)
 {
-	const char		*__function_name = "lld_graph_make";
+	const char			*__function_name = "lld_graph_make";
 
-	zbx_lld_graph_t		*graph = NULL;
-	char			*buffer = NULL;
-	struct zbx_json_parse	*jp_row = &lld_row->jp_row;
-	zbx_uint64_t		ymin_itemid, ymax_itemid;
+	zbx_lld_graph_t			*graph = NULL;
+	char				*buffer = NULL;
+	const struct zbx_json_parse	*jp_row = &lld_row->jp_row;
+	zbx_uint64_t			ymin_itemid, ymax_itemid;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
 
@@ -696,9 +697,9 @@ out:
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
-static void	lld_graphs_make(zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *graphs, zbx_vector_ptr_t *items,
+static void	lld_graphs_make(const zbx_vector_ptr_t *gitems_proto, zbx_vector_ptr_t *graphs, zbx_vector_ptr_t *items,
 		const char *name_proto, zbx_uint64_t ymin_itemid_proto, zbx_uint64_t ymax_itemid_proto,
-		zbx_vector_ptr_t *lld_rows)
+		const zbx_vector_ptr_t *lld_rows)
 {
 	int	i;
 
@@ -738,6 +739,11 @@ static void	lld_validate_graph_field(zbx_lld_graph_t *graph, char **field, char
 		*error = zbx_strdcatf(*error, "Cannot %s graph: value \"%s\" is too long.\n",
 				(0 != graph->graphid ? "update" : "create"), *field);
 	}
+	else if (ZBX_FLAG_LLD_GRAPH_UPDATE_NAME == flag && '\0' == **field)
+	{
+		*error = zbx_strdcatf(*error, "Cannot %s graph: name is empty.\n",
+				(0 != graph->graphid ? "update" : "create"));
+	}
 	else
 		return;
 
@@ -904,14 +910,25 @@ static void	lld_graphs_validate(zbx_uint64_t hostid, zbx_vector_ptr_t *graphs, c
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
 }
 
-static void	lld_graphs_save(zbx_uint64_t hostid, zbx_uint64_t parent_graphid, zbx_vector_ptr_t *graphs, int width,
+/******************************************************************************
+ *                                                                            *
+ * Function: lld_graphs_save                                                  *
+ *                                                                            *
+ * Purpose: add or update graphs in database based on discovery rule          *
+ *                                                                            *
+ * Return value: SUCCEED - if graphs were successfully saved or saving        *
+ *                         was not necessary                                  *
+ *               FAIL    - graphs cannot be saved                             *
+ *                                                                            *
+ ******************************************************************************/
+static int	lld_graphs_save(zbx_uint64_t hostid, zbx_uint64_t parent_graphid, zbx_vector_ptr_t *graphs, int width,
 		int height, double yaxismin, double yaxismax, unsigned char show_work_period,
 		unsigned char show_triggers, unsigned char graphtype, unsigned char show_legend, unsigned char show_3d,
 		double percent_left, double percent_right, unsigned char ymin_type, unsigned char ymax_type)
 {
 	const char		*__function_name = "lld_graphs_save";
 
-	int			i, j, new_graphs = 0, upd_graphs = 0, new_gitems = 0;
+	int			ret = SUCCEED, i, j, new_graphs = 0, upd_graphs = 0, new_gitems = 0;
 	zbx_lld_graph_t		*graph;
 	zbx_lld_gitem_t		*gitem;
 	zbx_vector_ptr_t	upd_gitems; 	/* the ordered list of graphs_items which will be updated */
@@ -967,7 +984,7 @@ static void	lld_graphs_save(zbx_uint64_t hostid, zbx_uint64_t parent_graphid, zb
 
 	DBbegin();
 
-	if (SUCCEED != DBlock_hostid(hostid))
+	if (SUCCEED != (ret = DBlock_hostid(hostid)))
 	{
 		/* the host was removed while processing lld rule */
 		DBrollback();
@@ -1250,6 +1267,8 @@ out:
 	zbx_vector_ptr_destroy(&upd_gitems);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+
+	return ret;
 }
 
 /******************************************************************************
@@ -1262,13 +1281,18 @@ out:
  *             agent   - [IN] discovery item identificator from database      *
  *             jp_data - [IN] received data                                   *
  *                                                                            *
+ * Return value: SUCCEED - if graphs were successfully added/updated or       *
+ *                         adding/updating was not necessary                  *
+ *               FAIL    - graphs cannot be added/updated                     *
+ *                                                                            *
  * Author: Alexander Vladishev                                                *
  *                                                                            *
  ******************************************************************************/
-void	lld_update_graphs(zbx_uint64_t hostid, zbx_uint64_t lld_ruleid, zbx_vector_ptr_t *lld_rows, char **error)
+int	lld_update_graphs(zbx_uint64_t hostid, zbx_uint64_t lld_ruleid, const zbx_vector_ptr_t *lld_rows, char **error)
 {
 	const char		*__function_name = "lld_update_graphs";
 
+	int			ret = SUCCEED;
 	DB_RESULT		result;
 	DB_ROW			row;
 	zbx_vector_ptr_t	graphs;
@@ -1293,7 +1317,7 @@ void	lld_update_graphs(zbx_uint64_t hostid, zbx_uint64_t lld_ruleid, zbx_vector_
 				" and id.parent_itemid=" ZBX_FS_UI64,
 			lld_ruleid);
 
-	while (NULL != (row = DBfetch(result)))
+	while (SUCCEED == ret && NULL != (row = DBfetch(result)))
 	{
 		zbx_uint64_t	parent_graphid, ymin_itemid_proto, ymax_itemid_proto;
 		const char	*name_proto;
@@ -1331,9 +1355,9 @@ void	lld_update_graphs(zbx_uint64_t hostid, zbx_uint64_t lld_ruleid, zbx_vector_
 		lld_graphs_make(&gitems_proto, &graphs, &items, name_proto, ymin_itemid_proto, ymax_itemid_proto,
 				lld_rows);
 		lld_graphs_validate(hostid, &graphs, error);
-		lld_graphs_save(hostid, parent_graphid, &graphs, width, height, yaxismin, yaxismax, show_work_period,
-				show_triggers, graphtype, show_legend, show_3d, percent_left, percent_right,
-				ymin_type, ymax_type);
+		ret = lld_graphs_save(hostid, parent_graphid, &graphs, width, height, yaxismin, yaxismax,
+				show_work_period, show_triggers, graphtype, show_legend, show_3d, percent_left,
+				percent_right, ymin_type, ymax_type);
 
 		lld_items_free(&items);
 		lld_gitems_free(&gitems_proto);
@@ -1346,4 +1370,6 @@ void	lld_update_graphs(zbx_uint64_t hostid, zbx_uint64_t lld_ruleid, zbx_vector_
 	zbx_vector_ptr_destroy(&graphs);
 
 	zabbix_log(LOG_LEVEL_DEBUG, "End of %s()", __function_name);
+
+	return ret;
 }
