@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,14 +26,15 @@
 
 static char	*zbx_regexp(const char *string, const char *pattern, int *len, int flags)
 {
-	char		*c = NULL;
-	static char	*old_pattern = NULL;
-	static int	old_flags;
-	static regex_t	re;
-	regmatch_t	match;
+	ZBX_THREAD_LOCAL static char	*old_pattern = NULL;
+	ZBX_THREAD_LOCAL static int	old_flags;
+	ZBX_THREAD_LOCAL static regex_t	re;
+
+	char				*c = NULL;
+	regmatch_t			match;
 
 	if (NULL != len)
-		*len = 0;
+		*len = FAIL;
 
 	if (NULL == string)
 		goto out;
@@ -55,6 +56,11 @@ compile:
 	}
 	else
 	{
+#ifdef _WINDOWS
+		/* the Windows gnuregex implementation does not correctly clean up */
+		/* allocated memory after regcomp() failure                        */
+		regfree(&re);
+#endif
 		zbx_free(old_pattern);
 		goto out;
 	}
@@ -66,6 +72,10 @@ execute:
 		if (NULL != len)
 			*len = match.rm_eo - match.rm_so;
 	}
+	else if (NULL != len)
+	{
+		*len = SUCCEED;
+	}
 out:
 	return c;
 }
@@ -184,22 +194,27 @@ out:
  *                                    is used as output value.                   *
  *            flags            - [IN] the regcomp() function flags.              *
  *                                    See regcomp() manual.                      *
+ *            out              - [OUT] the output value if the input string      *
+ *                                     matches the specified regular expression  *
+ *                                     or NULL otherwise                         *
  *                                                                               *
- * Return value: Allocated string containing output value if the input           *
- *               string matches the specified regular expression or NULL         *
- *               otherwise.                                                      *
+ * Return value: SUCCEED - the regular expression match was done                 *
+ *               FAIL    - invalid regular expression                            *
  *                                                                               *
  *********************************************************************************/
-static char	*regexp_sub(const char *string, const char *pattern, const char *output_template, int flags)
+static int	regexp_sub(const char *string, const char *pattern, const char *output_template, int flags, char **out)
 {
-	static regex_t	re;
-	regmatch_t	match[10];	/* up to 10 capture groups in regexp */
-	char		*ptr = NULL;
-	static char	*old_pattern = NULL;
-	static int	old_flags;
+	ZBX_THREAD_LOCAL static char	*old_pattern = NULL;
+	ZBX_THREAD_LOCAL static int	old_flags;
+	ZBX_THREAD_LOCAL static regex_t	re;
+
+	regmatch_t			match[10];	/* up to 10 capture groups in regexp */
 
 	if (NULL == string)
-		return NULL;
+	{
+		zbx_free(*out);
+		return SUCCEED;
+	}
 
 	if (NULL == output_template || '\0' == *output_template)
 		flags |= REG_NOSUB;
@@ -221,14 +236,21 @@ compile:
 	}
 	else
 	{
+#ifdef _WINDOWS
+		/* the Windows gnuregex implementation does not correctly clean up */
+		/* allocated memory after regcomp() failure                        */
+		regfree(&re);
+#endif
 		zbx_free(old_pattern);
-		return NULL;
+		return FAIL;
 	}
 execute:
-	if (0 == regexec(&re, string, ARRSIZE(match), match, 0))
-		ptr = regexp_sub_replace(string, output_template, match, ARRSIZE(match));
+	zbx_free(*out);
 
-	return ptr;
+	if (0 == regexec(&re, string, ARRSIZE(match), match, 0))
+		*out = regexp_sub_replace(string, output_template, match, ARRSIZE(match));
+
+	return SUCCEED;
 }
 
 /*********************************************************************************
@@ -245,17 +267,19 @@ execute:
  *                                    string is constructed from template by     *
  *                                    replacing \<n> sequences with the captured *
  *                                    regexp group.                              *
+ *            out              - [OUT] the output value if the input string      *
+ *                                     matches the specified regular expression  *
+ *                                     or NULL otherwise                         *
  *                                                                               *
- * Return value: Allocated string containing resulting value or NULL if          *
- *               the input string does not match the specified regular           *
- *               expression.                                                     *
+ * Return value: SUCCEED - the regular expression match was done                 *
+ *               FAIL    - invalid regular expression                            *
  *                                                                               *
  * Comments: This function performs case sensitive match                         *
  *                                                                               *
  *********************************************************************************/
-char	*zbx_regexp_sub(const char *string, const char *pattern, const char *output_template)
+int	zbx_regexp_sub(const char *string, const char *pattern, const char *output_template, char **out)
 {
-	return regexp_sub(string, pattern, output_template, REG_EXTENDED | REG_NEWLINE);
+	return regexp_sub(string, pattern, output_template, REG_EXTENDED | REG_NEWLINE, out);
 }
 
 /*********************************************************************************
@@ -266,9 +290,22 @@ char	*zbx_regexp_sub(const char *string, const char *pattern, const char *output
  *          multiline matches are accepted.                                      *
  *                                                                               *
  *********************************************************************************/
-char	*zbx_mregexp_sub(const char *string, const char *pattern, const char *output_template)
+int	zbx_mregexp_sub(const char *string, const char *pattern, const char *output_template, char **out)
 {
-	return regexp_sub(string, pattern, output_template, REG_EXTENDED);
+	return regexp_sub(string, pattern, output_template, REG_EXTENDED, out);
+}
+
+/*********************************************************************************
+ *                                                                               *
+ * Function: zbx_iregexp_sub                                                     *
+ *                                                                               *
+ * Purpose: This function is similar to zbx_regexp_sub() with exception that     *
+ *          case insensitive matches are accepted.                               *
+ *                                                                               *
+ *********************************************************************************/
+int	zbx_iregexp_sub(const char *string, const char *pattern, const char *output_template, char **out)
+{
+	return regexp_sub(string, pattern, output_template, REG_EXTENDED | REG_ICASE, out);
 }
 
 /******************************************************************************
@@ -294,7 +331,7 @@ void	zbx_regexp_clean_expressions(zbx_vector_ptr_t *expressions)
 		zbx_free(regexp);
 	}
 
-	expressions->values_num = 0;
+	zbx_vector_ptr_clear(expressions);
 }
 
 void	add_regexp_ex(zbx_vector_ptr_t *regexps, const char *name, const char *expression, int expression_type,
@@ -337,26 +374,43 @@ void	add_regexp_ex(zbx_vector_ptr_t *regexps, const char *name, const char *expr
  *                                    (substitution) is stored.                   *
  *                                    Specify NULL to skip output value creation. *
  *                                                                                *
- * Return value: SUCCEED - the string matches the specified regular expression    *
- *               FAIL    - the string does not match the specified regular        *
- *                         expression                                             *
+ * Return value: ZBX_REGEXP_MATCH    - the string matches the specified regular   *
+ *                                     expression                                 *
+ *               ZBX_REGEXP_NO_MATCH - the string does not match the regular      *
+ *                                     expression                                 *
+ *               FAIL                - the string is NULL or the specified        *
+ *                                     regular expression is invalid              *
  *                                                                                *
  **********************************************************************************/
 static int	regexp_match_ex_regsub(const char *string, const char *pattern, int case_sensitive,
 		const char *output_template, char **output)
 {
-	char	*ptr = NULL;
-	int	regexp_flags = REG_EXTENDED | REG_NEWLINE;
+	int	regexp_flags = REG_EXTENDED | REG_NEWLINE, ret;
 
 	if (ZBX_IGNORE_CASE == case_sensitive)
 		regexp_flags |= REG_ICASE;
 
 	if (NULL == output)
-		ptr = zbx_regexp(string, pattern, NULL, regexp_flags);
+	{
+		if (NULL == zbx_regexp(string, pattern, &ret, regexp_flags))
+		{
+			if (FAIL != ret)
+				ret = ZBX_REGEXP_NO_MATCH;
+		}
+		else
+			ret = ZBX_REGEXP_MATCH;
+	}
 	else
-		*output = ptr = regexp_sub(string, pattern, output_template, regexp_flags);
+	{
+		if (SUCCEED == regexp_sub(string, pattern, output_template, regexp_flags, output))
+		{
+			ret = (NULL != *output ? ZBX_REGEXP_MATCH : ZBX_REGEXP_NO_MATCH);
+		}
+		else
+			ret = FAIL;
+	}
 
-	return NULL != ptr ? SUCCEED : FAIL;
+	return ret;
 }
 
 /**********************************************************************************
@@ -371,8 +425,8 @@ static int	regexp_match_ex_regsub(const char *string, const char *pattern, int c
  *             case_sensitive  - [IN] ZBX_IGNORE_CASE - case insensitive search   *
  *                                    ZBX_CASE_SENSITIVE - case sensitive search  *
  *                                                                                *
- * Return value: SUCCEED - string contains the specified substring                *
- *               FAIL    - string does not contain the specified substring        *
+ * Return value: ZBX_REGEXP_MATCH    - string contains the specified substring    *
+ *               ZBX_REGEXP_NO_MATCH - string does not contain the substring      *
  *                                                                                *
  **********************************************************************************/
 static int	regexp_match_ex_substring(const char *string, const char *pattern, int case_sensitive)
@@ -389,7 +443,7 @@ static int	regexp_match_ex_substring(const char *string, const char *pattern, in
 			break;
 	}
 
-	return NULL != ptr ? SUCCEED : FAIL;
+	return (NULL != ptr ? ZBX_REGEXP_MATCH : ZBX_REGEXP_NO_MATCH);
 }
 
 /**********************************************************************************
@@ -406,16 +460,17 @@ static int	regexp_match_ex_substring(const char *string, const char *pattern, in
  *             delimiter       - [IN] the delimiter separating items in the       *
  *                                    substring list                              *
  *                                                                                *
- * Return value: SUCCEED - string contains a substring from the list              *
- *               FAIL    - string contains no substrings from the list            *
+ * Return value: ZBX_REGEXP_MATCH    - string contains a substring from the list  *
+ *               ZBX_REGEXP_NO_MATCH - string does not contain any substrings     *
+ *                                     from the list                              *
  *                                                                                *
  **********************************************************************************/
 static int	regexp_match_ex_substring_list(const char *string, char *pattern, int case_sensitive, char delimiter)
 {
-	int	ret = FAIL;
+	int	ret = ZBX_REGEXP_NO_MATCH;
 	char	*s, *c;
 
-	for (s = pattern; '\0' != *s && SUCCEED != ret;)
+	for (s = pattern; '\0' != *s && ZBX_REGEXP_MATCH != ret;)
 	{
 		if (NULL != (c = strchr(s, delimiter)))
 			*c = '\0';
@@ -460,56 +515,73 @@ static int	regexp_match_ex_substring_list(const char *string, char *pattern, int
  *                                    (substitution) is stored.                   *
  *                                    Specify NULL to skip output value creation. *
  *                                                                                *
- * Return value: SUCCEED - the string matches the specified regular expression    *
- *               FAIL    - the string does not match the specified regular        *
- *                         expression                                             *
+ * Return value: ZBX_REGEXP_MATCH    - the string matches the specified regular   *
+ *                                     expression                                 *
+ *               ZBX_REGEXP_NO_MATCH - the string does not match the specified    *
+ *                                     regular expression                         *
+ *               FAIL                - invalid regular expression                 *
  *                                                                                *
  * Comments: For regular expressions and global regular expressions with 'Result  *
- *           is TRUE' type the output_template substitution result is stored into *
- *           output variable. For the other global regular expression types the   *
- *           whole string is stored into output variable.                         *
+ *           is TRUE' type the 'output_template' substitution result is stored    *
+ *           into 'output' variable. For other global regular expression types    *
+ *           the whole string is stored into 'output' variable.                   *
  *                                                                                *
  **********************************************************************************/
-int	regexp_sub_ex(zbx_vector_ptr_t *regexps, const char *string, const char *pattern,
+int	regexp_sub_ex(const zbx_vector_ptr_t *regexps, const char *string, const char *pattern,
 		int case_sensitive, const char *output_template, char **output)
 {
 	int	i, ret = FAIL;
+	char	*output_accu;		/* accumulator for 'output' when looping over global regexp subexpressions */
 
 	if (NULL == pattern || '\0' == *pattern)
 	{
 		/* always match when no pattern is specified */
-		ret = SUCCEED;
+		ret = ZBX_REGEXP_MATCH;
 		goto out;
 	}
 
-	if ('@' != *pattern)
+	if ('@' != *pattern)				/* not a global regexp */
 	{
 		ret = regexp_match_ex_regsub(string, pattern, case_sensitive, output_template, output);
 		goto out;
 	}
 
 	pattern++;
+	output_accu = NULL;
 
-	for (i = 0; i < regexps->values_num; i++)
+	for (i = 0; i < regexps->values_num; i++)	/* loop over global regexp subexpressions */
 	{
-		zbx_expression_t	*regexp = regexps->values[i];
+		const zbx_expression_t	*regexp = regexps->values[i];
 
 		if (0 != strcmp(regexp->name, pattern))
 			continue;
 
-		ret = FAIL;
-
 		switch (regexp->expression_type)
 		{
 			case EXPRESSION_TYPE_TRUE:
-				ret = regexp_match_ex_regsub(string, regexp->expression, regexp->case_sensitive,
-						output_template, output);
+				if (NULL != output)
+				{
+					char	*output_tmp = NULL;
+
+					if (ZBX_REGEXP_MATCH == (ret = regexp_match_ex_regsub(string,
+							regexp->expression, regexp->case_sensitive, output_template,
+							&output_tmp)))
+					{
+						zbx_free(output_accu);
+						output_accu = output_tmp;
+					}
+				}
+				else
+				{
+					ret = regexp_match_ex_regsub(string, regexp->expression, regexp->case_sensitive,
+							NULL, NULL);
+				}
 				break;
 			case EXPRESSION_TYPE_FALSE:
 				ret = regexp_match_ex_regsub(string, regexp->expression, regexp->case_sensitive,
 						NULL, NULL);
-				/* invert output value */
-				ret = (SUCCEED == ret) ? FAIL : SUCCEED;
+				if (FAIL != ret)	/* invert output value */
+					ret = (ZBX_REGEXP_MATCH == ret ? ZBX_REGEXP_NO_MATCH : ZBX_REGEXP_MATCH);
 				break;
 			case EXPRESSION_TYPE_INCLUDED:
 				ret = regexp_match_ex_substring(string, regexp->expression, regexp->case_sensitive);
@@ -517,19 +589,31 @@ int	regexp_sub_ex(zbx_vector_ptr_t *regexps, const char *string, const char *pat
 			case EXPRESSION_TYPE_NOT_INCLUDED:
 				ret = regexp_match_ex_substring(string, regexp->expression, regexp->case_sensitive);
 				/* invert output value */
-				ret = (SUCCEED == ret) ? FAIL : SUCCEED;
+				ret = (ZBX_REGEXP_MATCH == ret ? ZBX_REGEXP_NO_MATCH : ZBX_REGEXP_MATCH);
 				break;
 			case EXPRESSION_TYPE_ANY_INCLUDED:
 				ret = regexp_match_ex_substring_list(string, regexp->expression, regexp->case_sensitive,
 						regexp->exp_delimiter);
 				break;
+			default:
+				THIS_SHOULD_NEVER_HAPPEN;
+				ret = FAIL;
 		}
 
-		if (FAIL == ret)
+		if (FAIL == ret || ZBX_REGEXP_NO_MATCH == ret)
+		{
+			zbx_free(output_accu);
 			break;
+		}
+	}
+
+	if (ZBX_REGEXP_MATCH == ret && NULL != output_accu)
+	{
+		*output = output_accu;
+		return ZBX_REGEXP_MATCH;
 	}
 out:
-	if (SUCCEED == ret && NULL != output && NULL == *output)
+	if (ZBX_REGEXP_MATCH == ret && NULL != output && NULL == *output)
 	{
 		/* Handle output value allocation for global regular expression types   */
 		/* that cannot perform output_template substitution (practically        */
@@ -542,7 +626,7 @@ out:
 	return ret;
 }
 
-int	regexp_match_ex(zbx_vector_ptr_t *regexps, const char *string, const char *pattern, int case_sensitive)
+int	regexp_match_ex(const zbx_vector_ptr_t *regexps, const char *string, const char *pattern, int case_sensitive)
 {
 	return regexp_sub_ex(regexps, string, pattern, case_sensitive, NULL, NULL);
 }
