@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,28 +25,19 @@
 
 #include "heart.h"
 #include "../servercomms.h"
+#include "../../libs/zbxcrypto/tls.h"
 
-extern unsigned char	process_type, daemon_type;
+extern unsigned char	process_type, program_type;
 extern int		server_num, process_num;
 
 /******************************************************************************
  *                                                                            *
  * Function: send_heartbeat                                                   *
  *                                                                            *
- * Purpose:                                                                   *
- *                                                                            *
- * Parameters:                                                                *
- *                                                                            *
- * Return value:                                                              *
- *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
- * Comments:                                                                  *
- *                                                                            *
  ******************************************************************************/
 static int	send_heartbeat(void)
 {
-	zbx_sock_t	sock;
+	zbx_socket_t	sock;
 	struct zbx_json	j;
 	int		ret = SUCCEED;
 	char		*error = NULL;
@@ -62,7 +53,8 @@ static int	send_heartbeat(void)
 
 	if (SUCCEED != put_data_to_server(&sock, &j, &error))
 	{
-		zabbix_log(LOG_LEVEL_WARNING, "sending heartbeat message to server failed: %s", error);
+		zabbix_log(LOG_LEVEL_WARNING, "cannot send heartbeat message to server at \"%s\": %s",
+				sock.peer, error);
 		ret = FAIL;
 	}
 
@@ -78,14 +70,6 @@ static int	send_heartbeat(void)
  *                                                                            *
  * Purpose: periodically send heartbeat message to the server                 *
  *                                                                            *
- * Parameters:                                                                *
- *                                                                            *
- * Return value:                                                              *
- *                                                                            *
- * Author: Alexander Vladishev                                                *
- *                                                                            *
- * Comments:                                                                  *
- *                                                                            *
  ******************************************************************************/
 ZBX_THREAD_ENTRY(heart_thread, args)
 {
@@ -100,15 +84,20 @@ ZBX_THREAD_ENTRY(heart_thread, args)
 	server_num = ((zbx_thread_args_t *)args)->server_num;
 	process_num = ((zbx_thread_args_t *)args)->process_num;
 
-	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_daemon_type_string(daemon_type),
+	zabbix_log(LOG_LEVEL_INFORMATION, "%s #%d started [%s #%d]", get_program_type_string(program_type),
 			server_num, get_process_type_string(process_type), process_num);
 
+#if defined(HAVE_POLARSSL) || defined(HAVE_GNUTLS) || defined(HAVE_OPENSSL)
+	zbx_tls_init_child();
+#endif
 	last_stat_time = time(NULL);
 
 	zbx_setproctitle("%s [sending heartbeat message]", get_process_type_string(process_type));
 
 	for (;;)
 	{
+		zbx_handle_log();
+
 		if (0 != sleeptime)
 		{
 			zbx_setproctitle("%s [sending heartbeat message %s in " ZBX_FS_DBL " sec, "
