@@ -1451,7 +1451,6 @@ static void	correlate_events_by_global_rules(zbx_vector_ptr_t *trigger_diff, zbx
 	{
 		if (FAIL != zbx_vector_uint64_bsearch(triggerids_lock, queue->objectid,
 				ZBX_DEFAULT_UINT64_COMPARE_FUNC))
-
 		{
 			/* trigger already locked by this process, add to locked triggerids */
 			zbx_vector_uint64_append(&triggerids, queue->objectid);
@@ -1706,11 +1705,7 @@ static void	update_trigger_changes(zbx_vector_ptr_t *trigger_diff)
 		}
 
 		if (EVENT_SOURCE_INTERNAL == event->source)
-		{
-			diff->lastchange = event->clock;
-			diff->flags |= ZBX_FLAGS_TRIGGER_DIFF_UPDATE_LASTCHANGE;
 			continue;
-		}
 
 		if (TRIGGER_VALUE_OK == event->value && 0 == (event->flags & ZBX_FLAGS_DB_EVENT_LINKED))
 		{
@@ -1728,8 +1723,9 @@ static void	update_trigger_changes(zbx_vector_ptr_t *trigger_diff)
 			diff->flags |= ZBX_FLAGS_TRIGGER_DIFF_UPDATE_PROBLEM_COUNT;
 		}
 
-		/* remember the clock of the last event in the case we must update trigger lastchange */
+		/* always update trigger last change whenever a trigger event has been created */
 		diff->lastchange = event->clock;
+		diff->flags |= ZBX_FLAGS_TRIGGER_DIFF_UPDATE_LASTCHANGE;
 	}
 
 	/* recalculate trigger value from problem_count and mark for updating if necessary */
@@ -1745,7 +1741,7 @@ static void	update_trigger_changes(zbx_vector_ptr_t *trigger_diff)
 		if (new_value != diff->value)
 		{
 			diff->value = new_value;
-			diff->flags |= (ZBX_FLAGS_TRIGGER_DIFF_UPDATE_VALUE | ZBX_FLAGS_TRIGGER_DIFF_UPDATE_LASTCHANGE);
+			diff->flags |= ZBX_FLAGS_TRIGGER_DIFF_UPDATE_VALUE;
 		}
 	}
 }
