@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2016 Zabbix SIA
+** Copyright (C) 2001-2017 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -48,7 +48,11 @@ void	recv_discovery_data(zbx_socket_t *sock, struct zbx_json_parse *jp)
 		goto out;
 	}
 
-	process_dhis_data(jp);
+	if (SUCCEED != (ret = process_dhis_data(jp, &error)))
+	{
+		zabbix_log(LOG_LEVEL_WARNING, "received invalid discovery data from proxy \"%s\" at \"%s\": %s",
+				host, sock->peer, error);
+	}
 out:
 	zbx_send_response(sock, ret, error, CONFIG_TIMEOUT);
 
@@ -70,7 +74,7 @@ void	send_discovery_data(zbx_socket_t *sock)
 
 	struct zbx_json	j;
 	zbx_uint64_t	lastid;
-	int		records, ret = FAIL;
+	int		ret = FAIL;
 	char		*error = NULL;
 
 	zabbix_log(LOG_LEVEL_DEBUG, "In %s()", __function_name);
@@ -85,7 +89,7 @@ void	send_discovery_data(zbx_socket_t *sock)
 
 	zbx_json_addarray(&j, ZBX_PROTO_TAG_DATA);
 
-	records = proxy_get_dhis_data(&j, &lastid);
+	proxy_get_dhis_data(&j, &lastid);
 
 	zbx_json_close(&j);
 
@@ -100,7 +104,7 @@ void	send_discovery_data(zbx_socket_t *sock)
 	if (SUCCEED != zbx_recv_response(sock, CONFIG_TIMEOUT, &error))
 		goto out;
 
-	if (0 != records)
+	if (0 != lastid)
 		proxy_set_dhis_lastid(lastid);
 
 	ret = SUCCEED;
